@@ -13,7 +13,7 @@
 use crate::error::{CudaResult, DropResult, ToResult};
 use crate::event::Event;
 use crate::function::{BlockSize, Function, GridSize};
-use crate::sys::{self as cuda, cudaError_enum, CUstream};
+use crate::sys::{self as cuda, CUstream};
 use std::ffi::c_void;
 use std::mem;
 use std::panic;
@@ -59,6 +59,10 @@ bitflags::bitflags! {
 pub struct Stream {
     inner: CUstream,
 }
+
+unsafe impl Send for Stream {}
+unsafe impl Sync for Stream {}
+
 impl Stream {
     /// Create a new stream with the given flags and optional priority.
     ///
@@ -103,21 +107,6 @@ impl Stream {
     }
 
     /// Return the flags which were used to create this stream.
-    ///
-    /// # Examples
-    ///
-    /// ```
-    /// # use cust::*;
-    /// # use std::error::Error;
-    /// # fn main() -> Result<(), Box<dyn Error>> {
-    /// # let _ctx = quick_init()?;
-    /// use cust::stream::{Stream, StreamFlags};
-    ///
-    /// let stream = Stream::new(StreamFlags::NON_BLOCKING, None)?;
-    /// assert_eq!(StreamFlags::NON_BLOCKING, stream.get_flags().unwrap());
-    /// # Ok(())
-    /// # }
-    /// ```
     pub fn get_flags(&self) -> CudaResult<StreamFlags> {
         unsafe {
             let mut bits = 0u32;
@@ -162,9 +151,6 @@ impl Stream {
     ///
     /// Callbacks must not make any CUDA API calls.
     ///
-    /// The callback will be passed a `CudaResult<()>` indicating the
-    /// current state of the device with `Ok(())` denoting normal operation.
-    ///
     /// # Examples
     ///
     /// ```
@@ -178,8 +164,8 @@ impl Stream {
     ///
     /// // ... queue up some work on the stream
     ///
-    /// stream.add_callback(Box::new(|status| {
-    ///     println!("Device status is {:?}", status);
+    /// stream.add_callback(Box::new(|| {
+    ///     println!("Work is done!");
     /// }));
     ///
     /// // ... queue up some more work on the stream
@@ -187,14 +173,13 @@ impl Stream {
     /// # }
     pub fn add_callback<T>(&self, callback: Box<T>) -> CudaResult<()>
     where
-        T: FnOnce(CudaResult<()>) + Send,
+        T: FnOnce() + Send,
     {
         unsafe {
-            cuda::cuStreamAddCallback(
+            cuda::cuLaunchHostFunc(
                 self.inner,
                 Some(callback_wrapper::<T>),
                 Box::into_raw(callback) as *mut c_void,
-                0,
             )
             .to_result()
         }
@@ -354,16 +339,13 @@ impl Drop for Stream {
         }
     }
 }
-unsafe extern "C" fn callback_wrapper<T>(
-    _stream: CUstream,
-    status: cudaError_enum,
-    callback: *mut c_void,
-) where
-    T: FnOnce(CudaResult<()>) + Send,
+unsafe extern "C" fn callback_wrapper<T>(callback: *mut c_void)
+where
+    T: FnOnce() + Send,
 {
     // Stop panics from unwinding across the FFI
     let _ = panic::catch_unwind(|| {
         let callback: Box<T> = Box::from_raw(callback as *mut T);
-        callback(status.to_result());
+        callback();
     });
 }
